// ---------------------------------------------------------------------------
// Import the packages used by this class
// ---------------------------------------------------------------------------
import javax.swing.*;
import javax.swing.event.*;

import java.awt.*;
import java.awt.event.*;

import java.io.*;
import java.util.*;

// Import our custom packages
import com.mcp.stock.portfolios.*;
import com.mcp.stock.portfolios.event.*;
import com.mcp.stock.internet.*;

// ---------------------------------------------------------------------------
// Import our other components
// ---------------------------------------------------------------------------
import StockTabPanel;
import PortfolioListPanel;
import MainMenu;


// ---------------------------------------------------------------------------
// StockApplication
// ---------------------------------------------------------------------------
public class StockApplication extends JFrame 
        implements ActionListener, ListSelectionListener, StockUpdateListener
{
    // -----------------------------------------------------------------------
    // Define a constant filename for our portfolio manager
    // -----------------------------------------------------------------------
    public final static String portfolioFile = new String( "Portfolios.txt" );
    
    
    // -----------------------------------------------------------------------
    // Create our PortfolioManager
    // -----------------------------------------------------------------------
    protected PortfolioManager portfolioManager = new PortfolioManager();

    
    // -----------------------------------------------------------------------
    // Create our MainMenu
    // -----------------------------------------------------------------------
    MainMenu mainMenu = new MainMenu();
    
    
    // -----------------------------------------------------------------------
    // Create our panels
    // -----------------------------------------------------------------------
    StockTabPanel stockTabPanel = new StockTabPanel();
    PortfolioListPanel portfolioListPanel = new PortfolioListPanel();


    // -----------------------------------------------------------------------
    // Constructor
    // -----------------------------------------------------------------------
    public StockApplication()
    {
        // Setup our title
        super( "Stock Tracker Application" );
        
        // Add our main menu
        setJMenuBar( mainMenu );
        
        // Add ourself as a listener to the menu
        mainMenu.addActionListener( this );
        
        // Add our components
        stockTabPanel = new StockTabPanel();
        portfolioListPanel = new PortfolioListPanel();
        
        getContentPane().add( stockTabPanel, BorderLayout.NORTH );
        getContentPane().add( portfolioListPanel, BorderLayout.SOUTH );
        
        // Set our size
        setSize( 680, 600 );
        
        // Add this StockApplication class as a StockUpdateListener
        portfolioManager.addStockUpdateListener( this );
        
        // Load our portfolio manager
        if( portfolioManager.loadAllPortfolios( portfolioFile ).booleanValue() == false )
        {
            // We don't have any porfolios yet so prompt the user for one
            Object obResult = new Object();
            String strNewPortfolio = new String();
            obResult = null;
            
            // Loop until we get a good value 
            while( (obResult == null) || (strNewPortfolio.length() == 0) )
            {
                // Prompt the user
                obResult = JOptionPane.showInputDialog( 
                    this,
                    "Welcome to QStock! Please enter a new portfolio name",
                    "QStock Startup",
                    JOptionPane.INFORMATION_MESSAGE);
                
                // See if the user pressed OK or CANCEL
                if( obResult != null )
                {
                    // User pressed OK, get the string return value
                    strNewPortfolio = ( String )obResult;
                }
            }
            
            // Add the new portfolio to the portfolio manager
            portfolioManager.addPortfolio( new Portfolio( strNewPortfolio) );
        }
        
        // Initialize our portfolio list panel with all of the 
        // portfolios in our portfolio manager
        for( int i=0; i<portfolioManager.getNumberOfPortfolios(); i++ )
        {
            portfolioListPanel.addPortfolio(
                    portfolioManager.getPortfolio( i ).getName() );
        }

        // Select the first item in the portfolio list
        portfolioListPanel.setSelectedIndex( 0 );
        
        // Add ourself as a listener to the PortfolioListPanel
        portfolioListPanel.addListSelectionListener( this );
        
        // Set the stock table's portfolio
        stockTabPanel.stockTableTabPanel.stockTablePanel.stockTableModel.setPortfolio( 
            portfolioManager.getPortfolio( 0 ) );
            
        // Set the graph panel's portfolio
        stockTabPanel.graphTabPanel.setPortfolio( portfolioManager.getPortfolio( 0 ) );
        
        // Make ourself visible
        setVisible( true );
        
        // Add a window listener to list for our window closing
        addWindowListener
        (
            new WindowAdapter() 
            {
                public void windowClosing( WindowEvent e ) 
                {
                    exitApp();
                }
            }
        );
    }
    
    
    // -----------------------------------------------------------------------
    // ListSelectionListener: valueChanged
    // -----------------------------------------------------------------------
    public void valueChanged(ListSelectionEvent e)  
    {
        // Ignore adjustment events
        if (e.getValueIsAdjusting())
        {
            return;    
        }
            
        // Get the list from the event
        JList theList = (JList)e.getSource();
        
        // Get the index of the selected item
        int index = theList.getSelectedIndex();
        
        // Get the name of the item in the list box
        String strName = ( String )theList.getModel().getElementAt( index );
        
        // This is the portfolio we are going to abstract for
        // this event
        Portfolio p = new Portfolio();
        
        // Retrieve the portfolio for this item
        try
        {
            p = portfolioManager.getPortfolio( strName );
        }
        catch( NoSuchElementException ex )
        {
            System.out.println( 
                "Cannot find " + 
                strName + 
                " in this portfolio! " + 
                ex.toString() );
            
            return;
        }
        
        // Set this portfolio on the table
        stockTabPanel.stockTableTabPanel.stockTablePanel.stockTableModel.setPortfolio( p );
        
        // Set this portfolio on the graph
        stockTabPanel.graphTabPanel.setPortfolio( p );
        
    }
    
    
    // -----------------------------------------------------------------------
    // StockUpdate Event Handler: portfolioComplete
    // -----------------------------------------------------------------------
    public void portfolioComplete( StockUpdateEvent e )
    {
    }
    
    
    // -----------------------------------------------------------------------
    // StockUpdate Event Handler: portfolioManagerComplete
    // -----------------------------------------------------------------------
    public void portfolioManagerComplete( StockUpdateEvent e )
    {
        stockTabPanel.stockTableTabPanel.stockTablePanel.stockTableModel.fireTableDataChanged();
    }
    
    
    // -----------------------------------------------------------------------
    // Action Event Handler: actionPerformed
    // -----------------------------------------------------------------------
    public void actionPerformed(ActionEvent e)
    {
        // -------------------------------------------------------------------
        // Extract the JMenuItem that generated this event
        // -------------------------------------------------------------------
        JMenuItem item = ( JMenuItem )e.getSource();
        
        
        // -------------------------------------------------------------------
        // Find our which JMenuItem generated this event
        // -------------------------------------------------------------------
        if( item == mainMenu.fileNewPortfolioManager )
        {
            System.out.println( "File->New->Portfolio Manager" );
        }
        
        // File -> New -> Portfolio
        else if( item == mainMenu.fileNewPortfolio )
        {
            // Debug output
            System.out.println( "File->New->Portfolio" );
            
            // Prompt the user
            Object obResult = JOptionPane.showInputDialog( 
                this,
                "Please enter a new portfolio name",
                "QStock Startup",
                JOptionPane.INFORMATION_MESSAGE);
            
            // See if the user pressed OK or CANCEL
            if( obResult == null )
            {
                return;
            }
            
            // Cast the Object to a String
            String strNewPortfolio = ( String )obResult;
            
            // Add the new portfolio to the portfolio manager
            portfolioManager.addPortfolio( new Portfolio( strNewPortfolio) );
            
            // Add the portfolio to the portfolio list
            portfolioListPanel.addPortfolio( strNewPortfolio );
            
        }
        
        // File -> New ->Stock
        else if( item == mainMenu.fileNewStock )
        {
            // Debug Output
            System.out.println( "File->New->Stock" );
            
            // Prompt the user
            Object obResult = JOptionPane.showInputDialog( 
                this,
                "Please enter a new stock symbol",
                "New Stock",
                JOptionPane.INFORMATION_MESSAGE);
            
            // See if the user pressed OK or CANCEL
            if( obResult == null )
            {
                // User pressed CANCEL
                return;
            }
            
            // Cast the Object to a String
            String strNewStock = ( String )obResult;
            
            if( strNewStock.length() == 0 )
            {
                // User didn't enter anything
                return;
            }
            
            // Find out which portfolio to add this stock to
            int nSelectedIndex = portfolioListPanel.listPortfolios.getSelectedIndex();
            String strPortfolioName = ( String )
                portfolioListPanel.listPortfolios.getModel().getElementAt
                ( 
                    nSelectedIndex 
                );
             
            // Get the portfolio
            Portfolio portfolio = portfolioManager.getPortfolio( strPortfolioName );
            
            // Add the new element to the portfolio
            portfolio.addStock( new StockData( strNewStock ) );
            
            // Tell the table to update itself
            stockTabPanel.stockTableTabPanel.stockTablePanel.stockTableModel.fireTableDataChanged();
        }
        else if( item == mainMenu.fileOpen )
        {
            System.out.println( "File->Open" );
        }
        else if ( item == mainMenu.fileClose )
        {
            System.out.println( "File->Close" );
        }
        else if ( item == mainMenu.fileSave )
        {
            System.out.println( "File->Save" );
        }
        else if ( item == mainMenu.fileSaveAs )
        {
            System.out.println( "File->Save As" );
        }
        else if ( item == mainMenu.fileSaveAll )
        {
            System.out.println( "File->Save All" );
        }
        else if ( item == mainMenu.filePrint )
        {
            System.out.println( "File->Print" );
        }
        else if ( item == mainMenu.fileExit )
        {
            System.out.println( "File->Exit" );
        }
        else if ( item == mainMenu.editCut )
        {
            System.out.println( "Edit->Cut" );
        }
        else if ( item == mainMenu.editCopy )
        {
            System.out.println( "Edit->Copy" );
        }
        else if ( item == mainMenu.editPaste )
        {
            System.out.println( "Edit->Paste" );
        }
        else if ( item == mainMenu.editDelete )
        {
            System.out.println( "Edit->Delete" );
        }
        else if ( item == mainMenu.portfoliosImport )
        {
            System.out.println( "Portfolios->Import" );
        }
        else if ( item == mainMenu.portfoliosExport )
        {
            System.out.println( "Portfolios->Export" );
        }
        else if ( item == mainMenu.quotesUpdate )
        {
            // Tell the PortfolioManager to update its stock prices
            portfolioManager.updatePortfolioStockPrices();
        }
        else if ( item == mainMenu.quotesTimerOptions )
        {
            System.out.println( "Quotes->Timer Options" );
        }
        else if ( item == mainMenu.historyGetQuote )
        {
            System.out.println( "History->Get Quote" );
        }
        else if ( item == mainMenu.historyEnableTracking )
        {
            System.out.println( "History->Enable Tracking" );
        }
        else if ( item == mainMenu.historyImport )
        {
            System.out.println( "History->Import" );
        }
        else if ( item == mainMenu.historyExport )
        {
            System.out.println( "History->Export" );
        }
        else if ( item == mainMenu.graphOptions )
        {
            System.out.println( "Graph->Options" );
        }
        else if ( item == mainMenu.helpContents )
        {
            System.out.println( "Help->Contents" );
        }
        else if ( item == mainMenu.helpAbout )
        {
            System.out.println( "Help->About" );
        }
        
    }
    
    protected void exitApp()
    {
        // Save our portfolios
        portfolioManager.saveAllPortfolios( portfolioFile );
        
        // Close our application
        System.exit( 0 );
    }
    
    
    // -----------------------------------------------------------------------
    // Main entry point into the StockApplication class
    // -----------------------------------------------------------------------
    public static void main( String[] args ) 
    {
        StockApplication app = new StockApplication();
    }

}